unit UfrmMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, ComObj, ActiveX, AutoCAD_TLB;

type
  TfrmMain = class(TForm)
    btnDrawTestPolyline: TButton;
    btnArcInfo: TButton;
    Bevel1: TBevel;
    Label1: TLabel;
    Bevel2: TBevel;
    btnDrawHugeBulge: TButton;
    Label2: TLabel;
    Label3: TLabel;
    Image1: TImage;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnDrawTestPolylineClick(Sender: TObject);
    procedure btnArcInfoClick(Sender: TObject);
    procedure btnDrawHugeBulgeClick(Sender: TObject);
  private
  public
  end;

var
  frmMain: TfrmMain;
  Acad: OleVariant;

implementation

{$R *.dfm}

uses
  Math;

{ Common Acad Functions }

function AcCalcDistance(P1, P2: OleVariant): Double;
var
  UBound1, UBound2: Integer;
begin
  UBound1:= VarArrayHighBound(P1, 1);
  UBound2:= VarArrayHighBound(P2, 1);
  if (UBound1 = 2) and (UBound2 = 2) then
    Result:= Sqrt(Sqr(P2[0] - P1[0]) + Sqr(P2[1] - P1[1]) + Sqr(P2[2] - P1[2]))
  else if ((UBound1 = 1) and (UBound2 = 1)) or ((UBound1 = 2) and (UBound2 = 1))  or ((UBound1 = 1) and (UBound2 = 2)) then
    Result:= Sqrt(Sqr(P2[0] - P1[0]) + Sqr(P2[1] - P1[1]))
  else raise
    EOleSysError.CreateFmt('', []);
end;

function AcPt(X, Y: Double; Z: Double = 0): OleVariant;
begin
  Result:= VarArrayCreate([0, 2], varDouble);
  Result[0]:= X;
  Result[1]:= Y;
  Result[2]:= Z;
end;

function AcCalcBulgeRadiusAndCenter(P1, P2: OleVariant; Bulge: Double;
  var Radius: Double; var Center: OleVariant): Boolean;
var
  D, Q, A: Double;
  X1, Y1, X2, Y2, X, Y: Double;
begin
  Result:= False;
  X1:= P1[0]; Y1:= P1[1];
  X2:= P2[0]; Y2:= P2[1];
  //  ,   P1  P2
  X:= (X1 + X2) / 2;
  Y:= (Y1 + Y2) / 2;
  //     - -    
  if (Bulge = 0) or (Bulge = 1) or (Bulge = -1) then
  begin
    Center:= AcPt(X, Y);
    Exit;
  end;
  D:= AcCalcDistance(P1, P2);
  Radius:= D / (2* Sin(ArcTan(Abs(Bulge)) * 2));
  Q:= Sqrt(Sqr(Radius) - Sqr(D / 2));
  A:= ArcTan(Abs(Bulge)) * 4; //    (0..2Pi)
  if Bulge > 0 then
  begin
    if A > Pi then
      Center:= AcPt(X - Q * (Y1 - Y2) / D, Y - Q * (X2 - X1) / D)
    else
      Center:= AcPt(X + Q * (Y1 - Y2) / D, Y + Q * (X2 - X1) / D);
  end else
  begin
    if A > Pi then
      Center:= AcPt(X + Q * (Y1 - Y2) / D, Y + Q * (X2 - X1) / D)
    else
      Center:= AcPt(X - Q * (Y1 - Y2) / D, Y - Q * (X2 - X1) / D);
  end;
  Result:= True;
end;

{ TfrmMain }

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Acad:= GetActiveOleObject('AutoCAD.Application');
  Caption:= Application.Title;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  VariantClear(Acad);
end;

procedure TfrmMain.btnDrawTestPolylineClick(Sender: TObject);
var
  pt, Points, Polyline: OleVariant;
begin
  if not VarIsClear(Acad) then
  begin
    Points:= VarArrayCreate([0, 29], VarDouble);
    Points[0]:= -3;
    Points[1]:= -2.5;
    Points[2]:= -2;
    Points[3]:= -2.5;
    Points[4]:= -1;
    Points[5]:= -0.5;
    Points[6]:= -1;
    Points[7]:= 0;
    Points[8]:= 0;
    Points[9]:= 0;
    Points[10]:= 0;
    Points[11]:= -1.5;
    Points[12]:= 1;
    Points[13]:= -1.5;
    Points[14]:= 1;
    Points[15]:= 0.5;
    Points[16]:= 2;
    Points[17]:= 2.5;
    Points[18]:= 0;
    Points[19]:= 2.5;
    Points[20]:= -1;
    Points[21]:= 2.5;
    Points[22]:= -2;
    Points[23]:= 2.5;
    Points[24]:= -3;
    Points[25]:= 2.5;
    Points[26]:= -4;
    Points[27]:= 2.5;
    Points[28]:= -4;
    Points[29]:= -2.5;
    Polyline:= Acad.ActiveDocument.ModelSpace.AddLightWeightPolyline(Points);
    Polyline.SetBulge(1, -0.7);
    Polyline.SetBulge(4, -0.6);
    Polyline.SetBulge(7, +0.7);
    Polyline.SetBulge(9, +1.5);
    Polyline.SetBulge(11, -1.5);
    Polyline.SetBulge(13, -0.5);
    //
    pt:= AcPt(5, 0);
    Points:= VarArrayCreate([0, 9], VarDouble);
    Points[0]:= pt[0];
    Points[1]:= pt[1];
    Points[2]:= pt[0] + 1;
    Points[3]:= pt[1];
    Points[4]:= pt[0] + 1;
    Points[5]:= pt[1] + 0.5;
    Points[6]:= pt[0] + 2;
    Points[7]:= pt[1] + 2.5;
    Points[8]:= pt[0] + 3;
    Points[9]:= pt[1] + 2.5;
    Polyline:= Acad.ActiveDocument.ModelSpace.AddLightWeightPolyline(Points);
    Polyline.SetBulge(2, -0.7);
    //
    Acad.Update;
  end;
end;

procedure TfrmMain.btnArcInfoClick(Sender: TObject);
var
  BasePnt: OleVariant;
  AcUtility, Entity: OleVariant;
  Obj: IDispatch;
  Coords, C, P1, P2: OleVariant;
  i, N, K, LBnd, HBnd: Longint;
  Bulge, R: Double;
  Flag: Boolean;
begin
  Flag:= False;
  AcUtility:= Acad.ActiveDocument.Utility;
  try //  
    AcUtility.GetEntity(Obj, BasePnt, #13' : ');
    Entity:= Obj;
  except
    Exit;
  end;
  if Entity.EntityType <> acPolylineLight then Exit;
  Coords:= Entity.Coordinates; //   2D-
  LBnd:= VarArrayLowBound(Coords, 1);
  HBnd:= VarArrayHighBound(Coords, 1);
  K:= ((HBnd - LBnd) - 1) div 2;
  if Entity.Closed then N:= K + 1 else N:= K;
  for i:= 0 to N - 1 do
  begin
    P1:= Entity.Coordinate[i];
    if i = K then
      P2:= Entity.Coordinate[0]
    else
      P2:= Entity.Coordinate[i + 1];
    Bulge:= Entity.GetBulge(i);
    if Abs(Bulge) > 1E-11 then
    begin
      AcCalcBulgeRadiusAndCenter(P1, P2, Bulge, R, C);
      C:= Acad.ActiveDocument.Utility.TranslateCoordinates(C, acOCS, acWorld, 0, Entity.Normal);
      //  ,    
      if not Flag then
      AcUtility.Prompt(Format(#13#10'%2s %6s %8s %8s %8s'#13#10,
        ['N', 'R', 'X', 'Y', 'B']));
      Flag:= True;
      //  
      AcUtility.Prompt(Format('%2d %8.4f %8.4f %8.4f %8.4f'#13#10,
        [i, R, Double(C[0]), Double(C[1]), Bulge]));
    end;
  end;
end;

procedure TfrmMain.btnDrawHugeBulgeClick(Sender: TObject);
var
  Points, Polyline: OleVariant;
begin
  if not VarIsClear(Acad) then
  begin
    if MessageBox(Handle,
       PChar('     ,    .'#13#10 +
             '        AutoCAD (   2008-)'#13#10 +
             '-        AutoCAD .'#13#10 +
             '  Bulge   (-1E-6)     500'#13#10 +
             '   AutoCAD !'),
       PChar(Application.Title),
       MB_YESNO or MB_ICONQUESTION) <> IDYES then Exit;
    Points:= VarArrayCreate([0, 7], VarDouble);
    Points[0]:=   0;       Points[1]:= 0;
    Points[2]:= 1E3;       Points[3]:= 0;
    Points[4]:= 1E9 + 1E3; Points[5]:= 0;
    Points[6]:= 1E9 + 2E3; Points[7]:= 0;
    Polyline:= Acad.ActiveDocument.ModelSpace.AddLightWeightPolyline(Points);
    Polyline.SetBulge(1, -1E-7);
    Polyline.Color:= acRed;
    Acad.Update;
  end;
end;

end.
